@@ -1,13 +1,15 @@
-#' Reduce results into a data.frame with all relevant information.
+#' @title Reduce results into a data.frame with all relevant information.
 #'
+#' @description
 #' Generates a \code{data.frame} with one row per job id. The columns are: ids of problem and algorithm
 #' (named \dQuote{prob} and \dQuote{algo}), one column per parameter of problem or algorithm (named by the parameter name),
 #' the replication number (named \dQuote{repl}) and all columns defined in the function to collect the values.
 #' Note that you cannot rely on the order of the columns.
-#' If a paramater does not have a setting for a certain job / experiment it is set to \code{NA}.
-#' Have a look at \code{\link{getResultVars}} if you want to use somethink like \code{\link{ddply}} on the
+#' If a parameter does not have a setting for a certain job / experiment it is set to \code{NA}.
+#' Have a look at \code{\link{getResultVars}} if you want to use something like \code{\link{ddply}} on the
 #' results.
 #'
+#' The rows are ordered as \code{ids} and named with \code{ids}, so one can easily index them.
 #'
 #' @param reg [\code{\link{ExperimentRegistry}}]\cr
 #'   Registry.
@@ -19,9 +21,9 @@
 #'   \code{NA} means all parts are loaded, which is the default.
 #' @param fun [\code{function(job, res, ...)}]\cr
 #'   Function to collect values from \code{job} and result \code{res} object, the latter from stored result file.
-#'   Must return an object which can be coerced to a \code{data.frame} (e.g. a \code{list}).
+#'   Must return a named object which can be coerced to a \code{data.frame} (e.g. a \code{list}).
 #'   Default is a function that simply returns \code{res} which may or may not work, depending on the type
-#'   of \code{res}.
+#'   of \code{res}. We recommend to always return a named list.
 #' @param ... [any]\cr
 #'   Additional arguments to \code{fun}.
 #' @param strings.as.factors [\code{logical(1)}]
@@ -52,8 +54,9 @@ reduceResultsExperiments = function(reg, ids, part = NA_character_, fun, ...,
       if (!is.list(impute.val) || !isProperlyNamed(impute.val))
         stop("Argument 'impute.val' must be a properly named list")
     } else {
-      if (length(ids) > length(done))
-        stopf("No results available for jobs with ids: %s", collapse(setdiff(ids, done)))
+      not.done = which(ids %nin% done)
+      if (length(not.done) > 0L)
+        stopf("No results available for jobs with ids: %s", collapse(not.done))
     }
   }
   BatchJobs:::checkPart(reg, part)
@@ -75,7 +78,7 @@ reduceResultsExperiments = function(reg, ids, part = NA_character_, fun, ...,
   impute = function(job, res, ...)
     impute.val
   getRow = function(j, reg, part, .fun, ...)
-    c(list(prob = j$prob.id), j$prob.pars, list(algo = j$algo.id), j$algo.pars, list(repl = j$repl),
+    c(list(id = j$id, prob = j$prob.id), j$prob.pars, list(algo = j$algo.id), j$algo.pars, list(repl = j$repl),
       .fun(j, BatchJobs:::getResult(reg, j$id, part), ...))
 
   aggr = data.frame()
@@ -87,12 +90,10 @@ reduceResultsExperiments = function(reg, ids, part = NA_character_, fun, ...,
 
   tryCatch({
     for(id.chunk in ids2) {
-      # FIXME: getJobs is inefficient here, we just want a data.frame
-      # FIXME: also check all other functions using getJobs / rbind.fill
       jobs = getJobs(reg, id.chunk, check.ids = FALSE)
       prob.pars = unique(c(prob.pars, unlist(lapply(jobs, function(j) names(j$prob.pars)))))
       algo.pars = unique(c(algo.pars, unlist(lapply(jobs, function(j) names(j$algo.pars)))))
-      # FIXME m/b use list2df instead of rbind.fill
+      # FIXME: m/b use convertListOfRowsToDataFrame instead of rbind.fill
       # -> major problem: how to deal with missing names in return value of fun?
       #    rbind.fill might not do the right thing here, also.
       id.chunk.done = id.chunk %in% done
@@ -104,8 +105,14 @@ reduceResultsExperiments = function(reg, ids, part = NA_character_, fun, ...,
   }, error = bar$error)
 
   aggr = convertDataFrameCols(aggr, chars.as.factor = strings.as.factors)
-  if (nrow(aggr))
-    aggr = setRowNames(cbind(id = ids, aggr), ids)
+  # name rows with ids so one can easily index
+  # THEN RESORT WRT TO IDS from call
+  # NB: in the for-loop above we potentially changed that order if we used imputing,
+  # see lines after id.chunk.done = ...
+  if (nrow(aggr) > 0L) {
+    aggr = setRowNames(aggr, aggr$id)
+    aggr = aggr[as.character(ids), ]
+  }
   aggr = addClasses(aggr, "ReducedResultsExperiments")
   attr(aggr, "prob.pars.names") = prob.pars
   attr(aggr, "algo.pars.names") = algo.pars
